/***************************************************************************
 * Oware computer move function module
 *
 *
 *
 ***************************************************************************/ 

#include <stdlib.h>
#include <stdbool.h>
#ifndef NP_RANDOMIZE
#include <fcntl.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#endif
#include "oware-ai.h"

#define NO_MOVE_LIST (0xFFFF)
#define NO_GRAD      (0xFFFF)
#define MIN_GRAD     (-1024)
#define MAX_GRAD     ( 1024)

static int move_list[15];

typedef struct Node {
    struct Node* parent;    /* link to previous position*/
#ifdef DEBUG
    struct Node* child;     /* link to current child position*/
#endif
    Board board;            /* board representation*/
    PossibleMoves pmoves;   /* List of all possible moves*/
    int moveIdx;            /* current child (index of currently evaluated move)*/
    int l;                  /* level deep.*/
    int grad;               /* rating for this position*/
    int bound_A, bound_B;   /* Node bounds for Alfa-Beta puring*/
    bool done;
} *pNode;



pNode evaluate_node (pNode current, int player) 
{
   current->grad = grade(&current->board, player);
   current->done = true;
   return current;
}

static pNode root;

#define cPlayer(node) ((player + node->l) %2)
#define my_node(node) (!(node->l %2))
static pNode go_down(const pNode current, pNode *next_free, int player) 
{
    pNode child;  

    /*If there is no list of possible moves get one */
    if (current->moveIdx == NO_MOVE_LIST)
    {
       current->pmoves = *getAllPossibleMoves(&current->board,
             cPlayer(current));
       current->moveIdx = 0;
    }

    /* Check if there are any possible moves left if not evaluate and return
     * this node */
    if (current->moveIdx == current->pmoves.count)
    {
       current->done = true;
       return current;
    }

    child = (*next_free)++;
    child->board = current->board;

    if (move(&child->board, cPlayer(current),
          current->pmoves.moves[current->moveIdx]) < 0)
    {
       printf("AI: failed move %d, player %d on board:\n",
             current->pmoves.moves[current->moveIdx],
             cPlayer(current));
       print_text(&child->board);
       exit(1);
    }

#ifdef DEBUG
    current->child = child;
    child->child   = NULL;
#endif

    child->l = current->l +1;
    child->moveIdx = NO_MOVE_LIST;
    child->done = false;
    child->parent = current;
    child->grad  = NO_GRAD;
    child->bound_B = current->bound_B;
    child->bound_A = current->bound_A;
    if (current->grad != NO_GRAD)
    {
       if (my_node(current) && current->grad > child->bound_A)
       {
          child->bound_A = current->grad;
       }
       if (!my_node(current) && current->grad < child->bound_B)
       {
          child->bound_B = current->grad;
       }
    }
    return child;
}

#ifndef NP_RANDOMIZE
static char random_byte(void)
{
#define B_SIZE (200)
   static int uran_fd = 0;
   static char buf[B_SIZE];
   static int idx;
   if (uran_fd == 0)
   {
      uran_fd = open("/dev/urandom", O_RDONLY);
      idx = B_SIZE;
   }
   if (idx == B_SIZE)
   {
      read(uran_fd, buf, B_SIZE);
      idx = 0;
   }
   return buf[idx++];
}
#endif

static pNode go_up(pNode current, pNode *next_free, int player)
{
    pNode parent;
    (*next_free)--;
    parent = current->parent;

    if (parent->done)
    {
       parent->moveIdx++;
       return parent;
    }
#ifndef NP_RANDOMIZE
    if (parent->l == 0)
    {
       /* Add some random value to the evaluated value of level one nodes. This
        * is done to avoid AI always choosing same moves for any given board */
       current->grad += random_byte()/96;
    }
#endif
    if (
          /* There is no grad yet */
          (parent->grad == NO_GRAD)
          ||
          /* It is our node, so we need max */
          (my_node(parent) && (current->grad > parent->grad ))
          ||
          /* It is not our node, so we need min */
          (!my_node(parent) && (current->grad < parent->grad))
       )
    { 
       move_list[parent->l] = parent->pmoves.moves[parent->moveIdx];
       parent->grad = current->grad;

#ifndef NP_ALPHABETA
       /* Alpha/Beta cutoff stop processing if the current result proves that parent
        * can not afffect best result*/
       if (!my_node(parent) && (parent->grad < parent->bound_A))
       {
          /*The current value of a mininimum perent node is already lower then
           * the alpha bound.  */
          parent->done = true;
       }
       else if (my_node(parent) && (parent->grad > parent->bound_B))
       {
          /*The current value of a maximum parent node is already higher then
           * the beta bound.  */
          parent->done = true;
       }
#endif
    }

#ifdef DEBUG
    {
       pNode c = root;
       do
       {
          printf("-%02d", c->pmoves.moves[c->moveIdx]);
          c = c->child;
       } while(c != current);
       printf(" l%d c:%03d p:%03d a%03d b%03d pd%d myc%d myp%d\n",
             current->l, current->grad, parent->grad, parent->bound_A, parent->bound_B, parent->done,
             my_node(current), my_node(parent));
    }
#endif
       
    parent->moveIdx++;
    return parent;
}


int ai_move(const Board* board, char player, int level)
{

    pNode next_free, current;

    root = (pNode)malloc( (level+1)*sizeof(struct Node) );
    if( !root ) printf("out of memory!"), exit(8);

    current = root;
    current->l = 0;
    current->done = false;
    current->grad = NO_GRAD;
    current->bound_A = MIN_GRAD;
    current->bound_B = MAX_GRAD;
    current->moveIdx = NO_MOVE_LIST;
    next_free = root + 1;

    current->board = *board;
    while(1)
    {
        if(!current->done && current->l < level) 
        {
            current = go_down(current, &next_free, player);
        }
        else if(!current->done && current->l == level) 
        { 
            evaluate_node(current, player);
        }
        else if(current != root)        
        {
            current = go_up(current, &next_free, player);
        }
        else
        {
            free(root);
            return move_list[0];
        }
    }
}
