#include "oware+.h"

int start_amount;

/*
 * Try doing a move
 *
 * returns 0 if the move is valid.  */
int move(Board* board, char player, int pit)
{
    int i, count;

    /* Check if choosen pit is correct */
    if (pit < 0
          || pit >= (player*6 +6)
          || pit <  (player*6))
       return -1; 

    /* Check if player has any stones on his side */
    for (i = player*6; i<player*6 +6; i++)
        if(board->pits[i]>0)
            break; 

    /* No stones, only 'empty' move possible. */
    if (i == 6*player+6)
        return 1;            

    /* invalid move, empty pit */
    if (board->pits[pit] < 1 )
       return -2;    

    count = board->pits[pit];
    board->pits[pit] = 0;               /*pick up stones*/
    while (count > 0)
    {
        pit++; pit%=12;                 /*next pit*/
        board->pits[pit]++; count--;    /*put one stone in next pit.*/
    }

    if (1 < board->pits[pit] && board->pits[pit] <= 3)
    {	/* move ended in pole with 1or2*/
        if(player && pit<6)
        {   /* on enemy site*/
            board->scoreP1 += board->pits[pit];
            board->pits[pit] = 0;
            while(board->pits[pit] <= 3 && pit >=0)
            {
                board->scoreP1 += board->pits[pit];
                board->pits[pit] = 0;
                pit--;
            }
        }	
        else if(!player && pit>=6)
        {
           /* So if player put his las stone on enemy's site to the pit 
              in which are less then 3 stones but some, he gets all the stone
              from that pit.  Moreover if the previous pit is on enemy site and
              contain less the 3 he grab the stones as well, and so on....  */
            board->scoreP0 += board->pits[pit];
            board->pits[pit] = 0;
            while(board->pits[pit] <= 3 && pit >=6){
                board->scoreP0 += board->pits[pit];
                board->pits[pit] = 0;
                pit--;
            }
        }
    }
    return 0;
}

/*
 * Test if game run to an end 
 * */
bool end_of_game(const Board* board)
{
    if (6*start_amount <= board->scoreP0
            || 6*start_amount <= board->scoreP1 )
        return true;
    else
        return false;
}

void reset(Board* board, int number)
{
    int i;
    for (i=0; i<12; i++)
       board->pits[i] = number;
    board->scoreP0=board->scoreP1=0;
}


void print_text(const Board* board)
{
    int i;
    printf("P0:  ");
    for (i=0;i<6; i++) printf("%02i  ", board->pits[i]);
    printf("\n");

    printf("P1:  ");
    for(i=0;i<6;i++) printf("%02i  ", board->pits[11-i]);

    printf("Scores: player0 -- %d, player1  -- %d \n\n",
            board->scoreP0, board->scoreP1);
}

int human_move(const Board* board, char player, int level) 
{
    int move;
    printf("Player's %d move: ", player);
    scanf("%d", &move );
    return move;
}

void game_control(move_f fp0, int l0,
      move_f fp1, int l1,
      print_f print)
{
    Board board;
    Board undo_list[50];
    int m, result, turn = 0;
    char player = 0;
    reset(&board, start_amount);
    while (!end_of_game(&board)) 
    {
        undo_list[turn%50] = board;
        print(&board);

        if (player)
           m = fp1(&board, player, l1);
        else
           m = fp0(&board, player, l0);

        if (m == UNDO)
        {
           if (turn >= 2)
           {
              turn -= 2;
              board = undo_list[turn%50];
           }
        }
        else
        {
            result = move(&board, player, m); 
            if(result >= 0) 
            {
                player = (++turn)%2;
            }
        }
    }
    print(&board);
}
