#include "sdl_display.h"
#include "sdl_event.h"


#define FONT_NAME_LINUX "FreeSans.ttf"
#define FONT_NAME_OSX   "Arial.ttf"
#define FONT_SIZE       (22)

static char* font_files[] = {
   "/usr/share/fonts/truetype/freefont/"FONT_NAME_LINUX,
   "/usr/share/fonts/truetype/"FONT_NAME_LINUX,
   "/usr/local/share/fonts/truetype/"FONT_NAME_LINUX,
   "/Library/Fonts/"FONT_NAME_OSX,
   "./"FONT_NAME_LINUX,
   "./"FONT_NAME_OSX,
   "oware.app/Contents/Resources/fonts/"FONT_NAME_OSX,
   NULL
};
static char* gfx_dirs[] = {
   "../gfx/",
   "oware.app/Contents/Resources/gfx/",
   ".",
   gfxdir,
   NULL
};


static SDL_Surface *stones[21];
static SDL_Surface *pointer_ok;
static SDL_Surface *pointer_nok;
static SDL_Surface *target;
SDL_Surface *screen;
SDL_Surface *bg;
TTF_Font *font;

static const Board* human_move_board;
static int human_move_player;
static int moveresult;
#define NO_MOVE (-1)

bool sdl_init()
{
   int i;
   char **dir;
   char filename[200];
   static bool done = false;

   if (done)
      return true;

   if(SDL_Init(SDL_INIT_EVERYTHING) < 0)
      return false;

   atexit(SDL_Quit);
   
   if (TTF_Init() < 0 )
      return false;
   
   for (i = 0; i < 21; i++)
   {
      for (dir = gfx_dirs; *dir; dir++)
      {
         sprintf(filename, "%s/64px-Mancala_hole_%d.png", *dir, i);
         if ((stones[i] = IMG_Load(filename)))
            break;
      }
   }
   for (dir = gfx_dirs; *dir; dir++)
   {
      sprintf(filename, "%s/64px-blur_hole_green.png", *dir);
      if ((pointer_ok = IMG_Load(filename)))
         break;
   }
   for (dir = gfx_dirs; *dir; dir++)
   {
      sprintf(filename, "%s/64px-blur_hole_red.png", *dir);
      if ((pointer_nok = IMG_Load(filename)))
         break;
   }
   for (dir = gfx_dirs; *dir; dir++)
   {
      sprintf(filename, "%s/64px-blur_target_red.png", *dir);
      if((target = IMG_Load(filename)))
         break;
   }
   for (dir = gfx_dirs; *dir; dir++)
   {
      sprintf(filename, "%s/384x128px_walnut_bg.png", *dir);
      if((bg = IMG_Load(filename)))
         break;
   }

   if (stones[i] == NULL || pointer_ok == NULL || target == NULL)
   {
      fprintf(stderr,
            "Couldn't load %s: %s\n", filename, SDL_GetError());
      return false;
   }

   for (dir = font_files; *dir; dir++)
   {
      sprintf(filename, "%s", *dir);
      if((font = TTF_OpenFont(filename, FONT_SIZE)))
         break;
   }
   if (font == NULL)
   {
      fprintf(stderr,
            "TTF open: %s\n", TTF_GetError());
      return false;
   }
   if((screen = SDL_SetVideoMode(384, 168, 32,
               SDL_DOUBLEBUF|SDL_HWSURFACE)) == NULL)
      return false;

   done = true;
   return true;
}


static SDL_Rect
pitNrtoXY(int pit)
{
   SDL_Rect dest;
   assert(pit >= 0 && pit <= 12);

   dest.h = 0;
   dest.w = 0;

   if (pit < 6)
   {
      dest.x = pit*64;
      dest.y = 64;
      return dest;
   } else {
      dest.x = (11-pit)*64;
      dest.y = 0;
      return dest;
   }
}

SDL_Surface*
getATocken(int count, Uint16* w, Uint16* h)
{
   SDL_Color text_color = {200, 128, 255};
   SDL_Color bg_color   = {250, 250, 250};
   SDL_Surface *number;
   SDL_Surface *image;
   SDL_Rect dest;
   char nr[3];
   if (count < 20)
      return stones[count];

   /* make a image with a text number if there is no
    * ready image to display */
   sprintf(nr, "%d", count);
   number = TTF_RenderText_Shaded(font, nr, text_color, bg_color);
   image = stones[20];
   *w = image->w;
   *h = image->h;
   dest.x = 15; dest.y = 15;
   dest.w = 49; dest.h = 49;
   SDL_BlitSurface(number, NULL, image, &dest);
   SDL_FreeSurface(number);
   return image;
}


void
sdl_draw(const Board* board)
{
   SDL_Surface *image;
   SDL_Surface *text;
   SDL_Color text_color = {255, 128, 255};
   SDL_Color bg_color   = {0, 0, 0};
   SDL_Rect dest;
   char text_buf[200];
   int i;
   dest.x = 0;
   dest.y = 0;

   if (end_of_game(board))
   {
      sprintf(text_buf, "Player %d Wins! (%d to %d)",
            board->scoreP0 > board->scoreP1 ? 1 : 2,
            board->scoreP0, board->scoreP1);
   } else {
      sprintf(text_buf, "Player 1: %d   Player 2: %d",
            board->scoreP0, board->scoreP1);
   }
   text = TTF_RenderText_Shaded(font,
         text_buf, text_color, bg_color);
   SDL_BlitSurface(bg, NULL, screen, &dest);

   for (i = 0; i < 12; i++ )
   {
      int count = board->pits[i];
      dest = pitNrtoXY(i);
      image = getATocken(count, &dest.w, &dest.h);
      SDL_BlitSurface(image, NULL, screen, &dest);
   }
   dest.y += 2*dest.h + 5;
   dest.x = 40;
   SDL_BlitSurface(text, NULL, screen, &dest);
}

static void onMouseDown(SDL_Event* Event)
{
   int x, y;
   int mX = Event->motion.x;
   int mY = Event->motion.y;

   switch(Event->button.button)
   {
      case SDL_BUTTON_LEFT:
         break;
      case SDL_BUTTON_RIGHT:
         return;
      case SDL_BUTTON_MIDDLE:
         return;
   }
   x = mX/64;
   y = mY/64;
   moveresult = y == 0 ? 11-x: x;
   return;
}

static void onKeyDown(SDL_Event* Event)
{
   switch(Event->key.keysym.sym)
   {
      case SDLK_u:
         moveresult = UNDO;
         break;
      default:
         break;
   }
   return;
}

static void
sdl_display_pointed (const Board* board, int p, int t)
{
   SDL_Rect dest;
   Board copy = *board;
   sdl_draw(board);

   dest = pitNrtoXY(p);
   dest.w = 64; dest.h = 64;
   if (move(&copy, human_move_player, p) >= 0)
   {
      SDL_BlitSurface(pointer_ok, NULL, screen, &dest);
   } else {
      SDL_BlitSurface(pointer_nok, NULL, screen, &dest);
   }
   dest = pitNrtoXY(t);
   dest.w = 64; dest.h = 64;
   SDL_BlitSurface(target, NULL, screen, &dest);
   SDL_Flip(screen);
}

static void
onMouseMove(SDL_Event *Event)
{
   int mX = Event->motion.x;
   int mY = Event->motion.y;
   int pointed, target;
   static int last_pointed;
   int x, y;

   x = mX/64;
   y = mY/64;
   pointed = y == 0 ? 11-x: x;

   if (pointed == last_pointed)
      return;

   last_pointed = pointed;
   target = (pointed + human_move_board->pits[pointed]) %12;
   sdl_display_pointed(human_move_board, pointed, target);
}

int
sdl_human_move(const Board* _board, char player, int level)
{
   human_move_board = _board;
   human_move_player = player;
   addEventHandler(SDL_MOUSEMOTION, onMouseMove);
   addEventHandler(SDL_MOUSEBUTTONDOWN, onMouseDown);
   addEventHandler(SDL_KEYDOWN, onKeyDown);

   moveresult = NO_MOVE;
   while(moveresult == NO_MOVE)
      handle_event();

   rmEventHandler(onMouseMove);
   rmEventHandler(onMouseDown);
   return moveresult;
}

void sdl_game_init(void)
{

}
void
sdl_display (const Board* board)
{
   sdl_draw(board);
   SDL_Flip(screen);
}


