#include <assert.h>
#include "sdl_display.h"
#include "sdl_event.h"


#define MAX_HANDLERS (20)
typedef struct {
   SDL_EventType    type;
   HandlerFunction  handler;
} EventHandler;

static EventHandler eventHandlerTable[MAX_HANDLERS];
int addEventHandler(SDL_EventType type, HandlerFunction handler)
{
   int i;

   for(i = 0; i < MAX_HANDLERS; i++)
   {
      if (!eventHandlerTable[i].handler)
      {
         eventHandlerTable[i].handler = handler;
         eventHandlerTable[i].type    = type;
         return 0;
      }
   }
   return (-1);
}

int rmEventHandler(HandlerFunction handler)
{
   int i;

   for(i = 0; i < MAX_HANDLERS; i++)
   {
      if (eventHandlerTable[i].handler == handler)
      {
         eventHandlerTable[i].handler = NULL;
         return 0;
      }
   }
   return (-1);
}


static void callHandlers(SDL_Event* event)
{
   int i;
   for(i = 0; i < MAX_HANDLERS; i++)
   {
      if (eventHandlerTable[i].type == event->type 
            && eventHandlerTable[i].handler)
      {
         eventHandlerTable[i].handler(event);
      }
   }
   return;
}

static void OnInputFocus()
{
}

static void OnInputBlur()
{
}

static void OnKeyDown(SDLKey sym, SDLMod mod, Uint16 unicode)
{
}

static void OnKeyUp(SDLKey sym, SDLMod mod, Uint16 unicode)
{
}

static void OnMouseFocus()
{
}

static void OnMouseBlur()
{
}


static void OnMinimize()
{
}

static void OnRestore()
{
}

static void OnResize(int w,int h)
{
}

static void OnExpose()
{
}

static void OnExit()
{
   exit(0);
}

static void OnUser(Uint8 type, int code, void* data1, void* data2)
{
}

static void event_handler(SDL_Event* Event)
{
   switch(Event->type)
   {
      case SDL_ACTIVEEVENT:
         switch(Event->active.state)
         {
            case SDL_APPMOUSEFOCUS:
               if ( Event->active.gain )       OnMouseFocus();
               else                            OnMouseBlur();
               break;
            case SDL_APPINPUTFOCUS:
               if ( Event->active.gain )       OnInputFocus();
               else                            OnInputBlur();
               break;
            case SDL_APPACTIVE:    
               if ( Event->active.gain )       OnRestore();
               else                            OnMinimize();
               break;
         }
         break;

      case SDL_KEYDOWN:
         OnKeyDown(Event->key.keysym.sym,
               Event->key.keysym.mod,
               Event->key.keysym.unicode);
         break;

      case SDL_KEYUP:
         OnKeyUp(Event->key.keysym.sym,
               Event->key.keysym.mod,
               Event->key.keysym.unicode);
         break;

      case SDL_MOUSEMOTION:
         break;

      case SDL_MOUSEBUTTONDOWN:
         break;

      case SDL_MOUSEBUTTONUP:
         break;

      case SDL_JOYAXISMOTION:
      case SDL_JOYBALLMOTION:
      case SDL_JOYHATMOTION:
      case SDL_JOYBUTTONDOWN:
      case SDL_JOYBUTTONUP:
         /*No joystic supported*/
         break;

      case SDL_QUIT:
         OnExit();
         break;

      case SDL_SYSWMEVENT:
         /*Ignore WM events*/
         break;

      case SDL_VIDEORESIZE:
         OnResize(Event->resize.w,Event->resize.h);
         break;

      case SDL_VIDEOEXPOSE:
         OnExpose();
         break;

      default:
         OnUser(Event->user.type,
               Event->user.code,
               Event->user.data1,
               Event->user.data2);
         break;
   }
   callHandlers(Event);
}

void
handle_event(void)
{
   SDL_Event Event;
   if (SDL_WaitEvent(&Event) == 0)
   {
      printf("SDL event error");
      return;
   }
   event_handler(&Event);
}

void
handle_events(void) 
{
   while(true)
      handle_event();
}

void
sdl_wait_exit(void) 
{
   while(true)
      handle_event();
}

