#include <SDL.h>
#include <SDL_video.h>
#include <unistd.h>

#include "oware+.h"
#include "oware-ai.h"
#include "sdl_display.h"
#include "sdl_event.h"

#define OK  (0)
#define NOK (1)
#define P0  (0)
#define P1  (1)

int test_interface()
{
    Board board;

   printf("\n\nTC %s\n", __func__);

    reset(&board, 2);
    print_text(&board);

    reset(&board, 3);
    print_text(&board);

    reset(&board, 4);
    print_text(&board);

    reset(&board, 5);
    print_text(&board);

    reset(&board, 6);
    print_text(&board);

    reset(&board, 16);

    print_text(&board);
    printf("%s: OK\n", __func__);
   return OK;
}


int test_sdl_display()
{
    int i;
    Board board;

    if (sdl_init() != true)
       return false;

    printf("\n\n%s\n", __func__);

    reset(&board, 0);
    for (i = 0; i <12; i++)
       board.pits[i] = i;
    sdl_display(&board);
    sleep(1);

    reset(&board, 24);
    sdl_display(&board);
    sleep(1);

    reset(&board, 56);
    sdl_display(&board);
    sleep(1);

    reset(&board, 16);
    sdl_display(&board);
    sleep(1);

    game_control(sdl_human_move, 0, ai_move, 5,
          &sdl_display);

    printf("%s: OK\n", __func__);
    return 0;
}


int test_sdl_move()
{
    Board board;
    int moveresult;

    printf("\n\n%s\n", __func__);
    if (sdl_init() != true)
       return false;
    printf("\n\n%s\n", __func__);

    reset(&board, 3);
    sdl_display(&board);
    sleep(1);
    moveresult = sdl_human_move(&board, 0, 7);
    printf("move: %d\n", moveresult);
    move(&board, 0, moveresult);
    sdl_display(&board);
    sleep(1);

    printf("%s: OK\n", __func__);
    return OK;
}


int test_move()
{
    Board board;

   printf("\n\nTC %s\n", __func__);

    reset(&board, 2);
    print_text(&board);

    printf("move by player 0 from pit 0:\n");
    move(&board, 0, 0);
    print_text(&board);
    printf("move by player 0 from pit 1:\n");
    move(&board, 0, 1);
    print_text(&board);
    printf("move by player 0 from pit 5:\n");
    move(&board, 0, 5);
    print_text(&board);

    printf("%s: OK\n", __func__);
    return 0;
}


int test_getAllPossibleMoves() 
{
   Board board;
   PossibleMoves *pmoves;
   int i;

   printf("\n\nTC %s\n", __func__);
   reset(&board, 2);
   print_text(&board);
   pmoves = getAllPossibleMoves(&board, P0);
   printf("possible moves P0:\n");
   for (i = 0; i < pmoves->count; i++)
   {
      printf("%d ", pmoves->moves[i]);
   }
   printf("\n");
   if (pmoves->count != 6)
      return NOK;

   pmoves = getAllPossibleMoves(&board, P1);
   printf("possible moves P1:\n");
   for (i = 0; i < pmoves->count; i++)
   {
      printf("%d ", pmoves->moves[i]);
   }
   printf("\n");
   if (pmoves->count != 6)
      return NOK;

   board.pits[0] = 0;
   board.pits[6] = 0;
   print_text(&board);

   pmoves = getAllPossibleMoves(&board, P0);
   printf("possible moves P0:\n");
   for (i = 0; i < pmoves->count; i++)
   {
      printf("%d ", pmoves->moves[i]);
   }
   printf("\n");
   if (pmoves->count != 5)
      return NOK;

   pmoves = getAllPossibleMoves(&board, P1);
   printf("possible moves P1:\n");
   for (i = 0; i < pmoves->count; i++)
   {
      printf("%d ", pmoves->moves[i]);
   }
   printf("\n");
   if (pmoves->count != 5)
      return NOK;

   printf("%s: OK\n", __func__);
   return OK;
}


int test_ai()
{
   Board board;
   int l;

   printf("\n\nTC %s\n", __func__);
   for (l = 1; l <= 12; l++)
   {
      reset(&board, 2);
      printf("best move l:%d: %d\n", l, ai_move(&board, 0, l));
   }

   printf("%s: OK\n", __func__);
   return OK;
}


int main(int argc, char* argv[])
{
   return ( OK
          | test_interface()
          | test_move()
          | test_getAllPossibleMoves()
          | test_ai()
          | test_sdl_move()
          | test_sdl_display()
          );
}
